-------------------------------------------------------------------------------
--   OCO Source Materials
--
--   Licensed Materials - Property of IBM
--
--   5724-C02
--
--   (C) Copyright IBM Corp. 2003  All Rights Reserved.
--
--   US Government Users Restricted Rights - Use, duplication or
--   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
-------------------------------------------------------------------------------
--    File Information: %W%
--    Revision: %I%
--    Date: %G% %U%
-------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
-- TWH_CDW 
--
-- Change the TWH_CDW below to the correct CDW database name, if different.
--
-- *** You must change _user below to the warehouse DB2 user id for the script to run ***
------------------------------------------------------------------------------------------
CONNECT TO TWH_CDW USER _user;
COMMIT;

----------------------------------------------------
-- Drop views objects
----------------------------------------------------;
DROP VIEW BWM.VE_COMP_ATTR_LONG; 
DROP VIEW BWM.VD_COMP_ATTR_LONG;
DROP VIEW BWM.VE_COMP_NAME_LONG;
DROP VIEW BWM.VD_COMP_NAME_LONG;
DROP VIEW BWM.VE_HOST;
DROP VIEW BWM.VD_HOST;
DROP VIEW BWM.VD_TX;
DROP VIEW BWM.VE_TX;
DROP VIEW BWM.VE_APP;
DROP VIEW BWM.VE_TX_ND;
DROP VIEW BWM.VE_TX_ND_MET;
DROP VIEW BWM.VD_TX_ND_MET;
DROP VIEW BWM.VE_STG_TX_ND_HR;

-----------------------------------------------------
-- Drop various tables
-----------------------------------------------------
DROP TABLE BWM.CENTR_LOOKUP;
DROP TABLE BWM.CUST_LOOKUP;

DROP TABLE BWM.COMP_ATTR_LONG;
DROP TABLE BWM.COMP_NAME_LONG;
DROP TABLE BWM.STG_CDW_TX_ND_HR;
DROP TABLE BWM.STG_D_TX_ND;
DROP TABLE BWM.STG_D_APP;
DROP TABLE BWM.STG_TX_ND_MET;


--------------------------------------------------
-- Drop any staging tables used in ETL1
--------------------------------------------------
--Staging tables for WTP 5.2
DROP TABLE BWM.STG_HOST;
DROP TABLE BWM.STG_APPLICATION;
DROP TABLE BWM.STG_USER;
DROP TABLE BWM.STG_TRANSACTION;
DROP TABLE BWM.STG_NODE;
DROP TABLE BWM.STG_RELATIONMAP;
DROP TABLE BWM.STG_AGGREG_DATA;
DROP TABLE BWM.STG_MGMTPOLICY;
DROP TABLE BWM.STG_PATTERN;
DROP TABLE BWM.STG_THRESHOLD;
DROP TABLE BWM.STG_TR;

------------------------------------------------------------------------------------------
-- Reset extract values
------------------------------------------------------------------------------------------
UPDATE TWG.EXTRACT_CONTROL
   SET
        EXTCTL_TO_INTSEQ = -1,
        EXTCTL_FROM_INTSEQ = -1
   WHERE EXTCTL_TARGET LIKE 'BWM%';

UPDATE TWG.EXTRACT_CONTROL
   SET
        EXTCTL_FROM_DTTM = '1970-01-01-00.00.00.000000',
        EXTCTL_TO_DTTM   = '9999-01-01-00.00.00.000000'
   WHERE EXTCTL_TARGET LIKE 'BWM%';

COMMIT;

------------------------------------------------------------------------------------------
-- Clear out BWM CDW data
------------------------------------------------------------------------------------------
DELETE FROM TWG.COMPRELN
   WHERE MSRC_CORR_CD = 'BWM';

-- Clear out relations where a BWM component is the comp_target_id in a SHARED relation.
DELETE FROM TWG.COMPRELN
   WHERE MSRC_CORR_CD = 'SHARED' AND
         COMP_TARGET_ID IN (SELECT COMP_ID FROM TWG.COMP WHERE MSRC_CORR_CD = 'BWM');

DELETE FROM TWG.COMP_EXT
   WHERE COMP_ID IN (SELECT COMP_ID FROM TWG.COMP WHERE MSRC_CORR_CD = 'BWM');

DELETE FROM TWG.COMPATTR
   WHERE MSRC_CORR_CD = 'BWM'  AND COMP_ID NOT IN (SELECT COMP_ID  FROM TWG.COMP WHERE MSRC_CORR_CD = 'SHARED');


DELETE FROM TWG.MSMT
   WHERE MSRC_CORR_CD = 'BWM';

-- This needs cleaning up if updates have been made to the Comp table.
-- or deletes from the comp table will be constrained.
DELETE FROM TWG.COMP_CHANGE
   WHERE COMP_ID IN (SELECT COMP_ID FROM TWG.COMP WHERE MSRC_CORR_CD = 'BWM');

-- Delete rows from the TWG.COMP in subsets to try to
-- mitigate the impact on transaction logging.
DELETE FROM TWG.COMP
   WHERE COMPTYP_CD = 'BWM_HOST';

DELETE FROM TWG.COMP
   WHERE COMPTYP_CD = 'BWM_TX_NODE';

DELETE FROM TWG.COMP
   WHERE COMPTYP_CD = 'BWM_TRANSACTION';

DELETE FROM TWG.COMP
   WHERE COMPTYP_CD = 'BWM_PROBE';

DELETE FROM TWG.COMP
   WHERE COMPTYP_CD = 'BWM_MGMT_POLICY';
   
-- re-create comp attr long table and views
CREATE TABLE BWM.COMP_ATTR_LONG(
     COMPATTR_ID INTEGER NOT NULL,
     COMP_ID INTEGER NOT NULL,
     COMPATTR_VAL VARCHAR(4000) NOT NULL,
     AttrTyp_Cd CHAR(17) NOT NULL,
     PRIMARY KEY (COMPATTR_ID,COMP_ID)
);

CREATE VIEW BWM.VE_COMP_ATTR_LONG (MAX_VALUE)
     AS SELECT MAX(A.COMPATTR_ID)
     FROM TWG.CUR_COMP C,
          BWM.COMP_ATTR_LONG A
     WHERE A.COMP_ID=C.COMP_ID;

CREATE VIEW BWM.VD_COMP_ATTR_LONG(
     ORIG_COMPATTR_ID,
     ORIG_COMP_ID,
     ATTRTYP_CD,
     COMPATTR_VAL,
     ATTR_STRT_DTTM,
     ATTR_END_DTTM)
AS SELECT
     A.COMPATTR_ID,
     A.COMP_ID,
     CA.ATTRTYP_CD,
     A.COMPATTR_VAL,
     CA.COMPATTR_STRT_DTTM,
     CA.COMPATTR_END_DTTM
FROM TWG.CUR_COMP C,
     TWG.CUR_COMPATTR CA,
     BWM.COMP_ATTR_LONG A
WHERE A.COMP_ID=C.COMP_ID AND A.COMPATTR_ID=CA.COMPATTR_ID;

--  create comp name long table and views
CREATE TABLE BWM.COMP_NAME_LONG (
     COMP_ID INTEGER NOT NULL,
     COMP_NM VARCHAR(4000) NOT NULL,
     COMP_DS VARCHAR(254),
     COMPTYP_CD VARCHAR(17),
     PRIMARY KEY(COMP_ID)
);

CREATE VIEW BWM.VE_COMP_NAME_LONG (MAX_VALUE)
     AS SELECT MAX(C.COMP_ID)
     FROM TWG.CUR_COMP C,
          BWM.COMP_NAME_LONG L
     WHERE C.COMPTYP_CD IN('BWM_TRANSACTION','BWM_TX_NODE','BWM_PROBE','J2EE_SERVER')
       AND C.COMP_ID=L.COMP_ID;

CREATE VIEW BWM.VD_COMP_NAME_LONG(
     ORIG_COMP_ID,
     COMP_NM,
     COMP_SHORT_NM,
     COMP_DS,
     COMPTYP_CD,
     COMP_STRT_DTTM,
     COMP_END_DTTM)
AS SELECT
     L.COMP_ID AS ORIG_COMP_ID,
     L.COMP_NM AS COMP_NM,
     C.COMP_NM AS COMP_SHORT_NM,
     L.COMP_DS AS COMP_DS,
     L.COMPTYP_CD AS COMPTYP_CD,
     C.COMP_STRT_DTTM AS COMP_STRT_DTTM,
     C.COMP_END_DTTM AS COMP_END_DTTM
FROM TWG.CUR_COMP C,
     BWM.COMP_NAME_LONG L
WHERE C.COMPTYP_CD IN ('BWM_TRANSACTION','BWM_TX_NODE','BWM_PROBE','J2EE_SERVER')
     AND C.COMP_ID=L.COMP_ID;

-------------------------------------------------------------
-- Re-create the lookup tables for customer and centre
-------------------------------------------------------------
-----------------------------
-- lookup table for customer
-----------------------------
create table bwm.cust_lookup (
   cust_id  integer,
   value    char(32)
);

--------------------------
-- lookup table for centr
--------------------------
create table bwm.centr_lookup (
    centr_cd    char(6),
    value       char(32)
);

--Insert data into lookup tables to setup a single customer/center environment.
declare global temporary table bwm_cust_lookup (
   cust_id  integer,
   value    char(32)
)
with replace on commit preserve rows not logged
;

insert into session.bwm_cust_lookup values (1, '@');

INSERT INTO BWM.cust_lookup
SELECT * FROM session.bwm_cust_lookup s
WHERE NOT EXISTS (
   SELECT 1 FROM
   BWM.cust_lookup t WHERE
   t.cust_id = s.cust_id
);

declare global temporary table bwm_centr_lookup (
    centr_cd    char(3),
    value       char(32)
)
with replace on commit preserve rows not logged
;

insert into session.bwm_centr_lookup values ('CDW', '@');

INSERT INTO BWM.centr_lookup
SELECT * FROM session.bwm_centr_lookup s
WHERE NOT EXISTS (
   SELECT 1 FROM
   BWM.centr_lookup t WHERE
   t.centr_cd = s.centr_cd
);

---------------------
--- HOST TABLES/VIEWS
---------------------

-- VE VIEW
CREATE VIEW BWM.VE_HOST (MAX_VALUE) AS
 SELECT MAX(C.COMP_ID)
     FROM TWG.CUR_COMP C
     WHERE C.COMPTYP_CD IN ('IP_HOST','IP_INTERFACE','BWM_HOST')
;

-- VD VIEW
CREATE VIEW BWM.VD_HOST(
    ORIG_HOST_ID,
    HOST_NM,
    LAST_IP_ADDRESS,
    CUST_ID,
    CUST_NM,
    CENTR_NM)
    AS
 SELECT
  C1.COMP_ID AS ORIG_HOST_ID,
  C1.COMP_NM AS HOST_NM,
  VALUE(MIN(C2.COMPATTR_VAL),'N/A') AS LAST_IP_ADDRESS,
  CU.CUST_ID AS CUST_ID,
  CU.CUST_NM AS CUST_NM,
  CE.CENTR_NM AS CENTR_NM
 FROM
  TWG.CUR_COMP C1 LEFT OUTER JOIN TWG.CUR_COMPATTR C2
   ON C2.COMP_ID=C1.COMP_ID AND C2.ATTRTYP_CD='LAST_IP_ADDRESS',
  TWG.CUST CU,
  TWG.CENTR CE
 WHERE
  CU.CUST_ID=C1.CUST_ID AND CE.CENTR_CD=C1.CENTR_CD AND
  C1.COMPTYP_CD IN ('IP_HOST','BWM_HOST')
 GROUP BY
  C1.COMP_ID, C1.COMP_NM, CU.CUST_ID, CU.CUST_NM, CE.CENTR_NM
UNION
 SELECT
  C1.COMP_ID AS ORIG_HOST_ID,
  'N/A' AS HOST_NM,
  C1.COMP_NM AS LAST_IP_ADDRESS,
  CU.CUST_ID AS CUST_ID,
  CU.CUST_NM AS CUST_NM,
  CE.CENTR_NM AS CENTR_NM
 FROM
  TWG.CUR_COMP C1,
  TWG.CUST CU,
  TWG.CENTR CE
 WHERE
  CU.CUST_ID=C1.CUST_ID AND CE.CENTR_CD=C1.CENTR_CD AND
  C1.COMPTYP_CD IN ('IP_INTERFACE')
 GROUP BY
  C1.COMP_ID, C1.COMP_NM, CU.CUST_ID, CU.CUST_NM, CE.CENTR_NM
;

-- VE VIEW
CREATE VIEW BWM.VE_APP (MAX_VALUE) AS
 SELECT MAX(C.COMP_ID)
    FROM TWG.CUR_COMP C
    WHERE C.COMPTYP_CD IN ('J2EE_SERVER','BWM_PROBE')
    ;

--VE VIEW
CREATE VIEW BWM.VE_STG_TX_ND_HR (MAX_VALUE) AS
 SELECT MAX(M.MSMT_ID)
    FROM TWG.CUR_COMP C, TWG.MSMT M
    WHERE C.COMPTYP_CD='BWM_TX_NODE' AND C.COMP_ID=M.COMP_ID
;


---------------------
--- T_TX TABLES/VIEWS
---------------------

-- VE VIEW
CREATE VIEW BWM.VE_TX (MAX_VALUE) AS
 SELECT MAX(C.COMP_ID)
    FROM TWG.CUR_COMP C
    WHERE C.COMPTYP_CD IN ('BWM_TRANSACTION')
    ;

-- VD VIEW
CREATE VIEW BWM.VD_TX(
    ORIG_TX_ID,
    TX_NAME,
    TX_DESCRIPTION,
    URL_PROTOCOL,
    WEBSITE,
    WEBSITE_PATH,
    WEBSITE_QUERY)
    AS
 SELECT
  C0.COMP_ID AS ORIG_TX_ID,
  C0.COMP_NM AS TX_NAME,
  VALUE(MIN(C0.COMP_DS),' ') AS TX_DESCRIPTION,
  VALUE(MIN(CA1.COMPATTR_VAL),'N/A') AS URL_PROTOCOL,
  VALUE(MIN(CA2.COMPATTR_VAL),'N/A') AS WEBSITE,
  VALUE(MIN(CA3.COMPATTR_VAL),'N/A') AS WEBSITE_PATH,
  VALUE(MIN(CA4.COMPATTR_VAL),'N/A') AS WEBSITE_QUERY
 FROM
  TWG.CUR_COMP C0,
  TWG.CUR_COMP C1 LEFT OUTER JOIN TWG.CUR_COMPATTR CA1
   ON CA1.COMP_ID=C1.COMP_ID AND CA1.ATTRTYP_CD='URL_PROTOCOL',
  TWG.CUR_COMP C2 LEFT OUTER JOIN TWG.CUR_COMPATTR CA2
   ON CA2.COMP_ID=C2.COMP_ID AND CA2.ATTRTYP_CD='WEBSITE',
  TWG.CUR_COMP C3 LEFT OUTER JOIN TWG.CUR_COMPATTR CA3
   ON CA3.COMP_ID=C3.COMP_ID AND CA3.ATTRTYP_CD='WEBSITE_PATH',
  TWG.CUR_COMP C4 LEFT OUTER JOIN TWG.CUR_COMPATTR CA4
   ON CA4.COMP_ID=C4.COMP_ID AND CA4.ATTRTYP_CD='WEBSITE_QUERY',
  TWG.COMPRELN R
 WHERE
  C0.COMPTYP_CD IN ('BWM_TRANSACTION') AND
  C1.COMPTYP_CD IN ('BWM_TX_NODE') AND
  C1.COMP_ID=C2.COMP_ID AND
  C1.COMP_ID=C3.COMP_ID AND
  C1.COMP_ID=C4.COMP_ID AND
  R.RELNTYP_CD='INSTOF' AND
  R.COMP_SOURCE_ID=C1.COMP_ID AND
  R.COMP_TARGET_ID=C0.COMP_ID
 GROUP BY
  C0.COMP_ID,C0.COMP_NM,C0.COMP_DS
;

--------------------
---  STG_TX_ND_MET
--------------------
CREATE TABLE BWM.STG_TX_ND_MET(
    INSERT_DTTM TIMESTAMP,
    METRIC_ID INTEGER NOT NULL,
    MET_CATEGORY VARCHAR(254),
    MET_DESC VARCHAR(254),
    MET_NAME VARCHAR(254) NOT NULL,
    MET_UNITS VARCHAR(254),
    MIN_EXISTS CHAR(1) NOT NULL,
    MAX_EXISTS CHAR(1) NOT NULL,
    AVG_EXISTS CHAR(1) NOT NULL,
    TOTAL_EXISTS CHAR(1) NOT NULL,
    MSRC_NM VARCHAR(254),
    PRIMARY KEY(METRIC_ID))
    ;

-- VE VIEW
CREATE VIEW BWM.VE_TX_ND_MET (MAX_VALUE) AS
 SELECT MAX(METRIC_ID)
    FROM BWM.STG_TX_ND_MET
;

-- VD VIEW
CREATE VIEW BWM.VD_TX_ND_MET(
    ORIG_METRIC_ID,
    MET_CATEGORY,
    MET_DESC,
    MET_NAME,
    MET_UNITS,
    MIN_EXISTS,
    MAX_EXISTS,
    AVG_EXISTS,
    TOTAL_EXISTS,
    MSRC_NM)
    AS
 SELECT
  D.METRIC_ID AS ORIG_METRIC_ID,
  D.MET_CATEGORY AS MET_CATEGORY,
  D.MET_DESC AS MET_DESC,
  D.MET_NAME AS MET_NAME,
  D.MET_UNITS AS MET_UNITS,
  D.MIN_EXISTS AS MIN_EXISTS,
  D.MAX_EXISTS AS MAX_EXISTS,
  D.AVG_EXISTS AS AVG_EXISTS,
  D.TOTAL_EXISTS AS TOTAL_EXISTS,
  D.MSRC_NM AS MSRC_NM
 FROM BWM.STG_TX_ND_MET D
;

-- VE VIEW
CREATE VIEW BWM.VE_TX_ND (MAX_VALUE) AS
 SELECT MAX(C.COMP_ID)
    FROM TWG.CUR_COMP C
    WHERE C.COMPTYP_CD='BWM_TX_NODE'
;

-------------------------------------------------------------------------------
-- temporary tables indicating which measurement is in the group MIN_E,
-- MAX_E, AVG_E, TOT_E for the BWM DataMart
-------------------------------------------------------------------------------
declare global temporary table temp_min_exists (
 msmttyp_id	integer,
 min_exists     char(1)
)
with replace on commit preserve rows not logged
;

insert into session.temp_min_exists
select
 mb.msmttyp_id,
 'Y' as min_exists
from
 twg.mgrpmbr mb
where
 mb.mgrptyp_cd = 'GROUP' and
 mb.mgrp_cd='MIN_E'
;


-------------------------------------------------------------------------------
-- select * from session.temp_min_exists;
-------------------------------------------------------------------------------
declare global temporary table temp_max_exists (
 msmttyp_id	integer,
 max_exists     char(1)
)
with replace on commit preserve rows not logged
;

insert into session.temp_max_exists
select
 mb.msmttyp_id,
 'Y' as max_exists
from
 twg.mgrpmbr mb
where
 mb.mgrptyp_cd = 'GROUP' and
 mb.mgrp_cd='MAX_E'
;

declare global temporary table temp_avg_exists (
 msmttyp_id	integer,
 avg_exists     char(1)
)
with replace on commit preserve rows not logged
;

insert into session.temp_avg_exists
select
 mb.msmttyp_id,
 'Y' as avg_exists
from
 twg.mgrpmbr mb
where
 mb.mgrptyp_cd = 'GROUP' and
 mb.mgrp_cd='AVG_E'
;

-------------------------------------------------------------------------------
-- select * from session.temp_avg_exists;
-------------------------------------------------------------------------------
declare global temporary table temp_tot_exists (
 msmttyp_id	integer,
 tot_exists     char(1)
)
with replace on commit preserve rows not logged
;

insert into session.temp_tot_exists
select
 mb.msmttyp_id,
 'Y' as tot_exists
from
 twg.mgrpmbr mb
where
 mb.mgrptyp_cd = 'GROUP' and
 mb.mgrp_cd='TOT_E'
;

-------------------------------------------------------------------------------
-- insert the  measurement types in the metrics dimensions for
-- the component BWM_TX_NODE
-------------------------------------------------------------------------------
insert into BWM.STG_TX_ND_MET
(
insert_dttm,
metric_id,
met_category,
met_name,
met_desc,
met_units,
min_exists,
max_exists,
avg_exists,
total_exists,
msrc_nm)
select
 current timestamp - current timezone as insert_dttm,
 mt.msmttyp_id as metric_id,
 'Not used' AS met_category,
 mt.msmttyp_nm as met_name,
 mt.msmttyp_ds as met_desc,
 mt.munit_cd as met_units ,
 value(min.min_exists,'N') as min_exists,
 value(max.max_exists,'N') as max_exists,
 value(avg.avg_exists,'N') as avg_exists,
 value(tot.tot_exists,'N') as total_exists,
 s.msrc_nm as msrc_nm
from
 twg.msmttyp mt left outer join session.temp_min_exists min
 on mt.msmttyp_id=min.msmttyp_id,
 twg.msmttyp mt2 left outer join session.temp_max_exists max
 on mt2.msmttyp_id=max.msmttyp_id,
 twg.msmttyp mt3 left outer join session.temp_avg_exists avg
 on mt3.msmttyp_id=avg.msmttyp_id,
 twg.msmttyp mt4 left outer join session.temp_tot_exists tot
 on mt4.msmttyp_id=tot.msmttyp_id,
 twg.msmtrul mr,
 twg.msrc s
where
 mt.msmttyp_id = mr.msmttyp_id and
 mt.msrc_cd IN ('BWM','MODEL1') and
 s.msrc_cd=mt.msrc_cd and
 mr.comptyp_cd = 'BWM_TX_NODE' and
 mt.msmttyp_id = mt2.msmttyp_id and
 mt.msmttyp_id = mt3.msmttyp_id and
 mt.msmttyp_id = mt4.msmttyp_id and
 not exists (select 1 from BWM.STG_TX_ND_MET m
        where
        m.metric_id = mt.msmttyp_id)
group by
 mt.msmttyp_id, mt.msmttyp_nm, mt.msmttyp_ds, mt.munit_cd,
 min_exists, avg_exists, tot_exists, max_exists, s.msrc_nm
;


-- End of script





